//***************************克哈RA2平衡小车***************************************//
#include "controller.h"
#include "imu.h"
#include "mpu6050.h"
#include "imath.h"

_OUT_Motor Motor1 = {0};
_OUT_Motor Motor2 = {0};

/* PID参数 */
//不同的电机改不同的PID参数根据电机调整

#if 1 /* product motor */
_PID_CONTROL vel = {130.0f, 0.65f, 0.0f};       //{130.0f, 0.65f, 0.0f};
_PID_CONTROL bal = {300.0f, 0.0f , 20.0f};      //{150.0f, 0.0f , 20.0f};
_PID_CONTROL tur = {5.0f,  0.0f,  10.0f};      //{10.0f,  0.0f,  15.5f};

#endif


_OUT_CONTROL ctr = {0};
_REMOTE_Data remote = {0};

/*
 * 函数名：CarSpeedCtrlForwardOrBackward
 * 描述  ：遥控时前进或者后退的速度更改
 * 输入  ：ChangeStep增加或者减少的阶梯步数(改变前后运行速度的快慢程度)
 * 返回  ：处理及限幅之后的前后方向速度更改阶梯步数
 */
//int CarSpeedCtrlForwardOrBackward(int ChangeStep)
//{
//    static int carUpOrDownOut = 40;                                 //开机上电默认前进后退速度
//
//    ForceIntLimit(&carUpOrDownOut,10,70);
//    return carUpOrDownOut;                                          //限幅
//}

/*
 * 函数名：CarSpeedCtrlLeftOrRight
 * 描述  ：更改遥控时左右转向/旋转的速度快慢
 * 输入  ：ChangeStep增加或者减少的阶梯步数(改变前后运行速度的快慢程度)
 * 返回  ：处理及限幅之后的左右/旋转方向速度更改阶梯步数
 */
//int CarSpeedCtrlLeftOrRight(int ChangeStep)
//{
//    static int carLeftOrRightOut = 2500;                            //开机上电转向的初始控制量输出
//
//    ForceIntLimit(&carLeftOrRightOut,1000,4000);
//    return carLeftOrRightOut;                                       //限幅
//}


/*
 * 描述  ：模式控制
 * 返回  ：预期速度
 */
float remoteCarForwardOrBackward(uint8_t Mode)//Mode=0平衡模式  Mode=1跟随模式
{
    float TargetVel;

    if( Mode == 0)
    {
        TargetVel = 0;
    }
    else if( Mode == 1)                                  //超声波跟随模式
    {
        if(( dis <= 10) && ( dis > 0 ))                 //距离小于10CM小车后退
        {
            TargetVel = -25;                                            //后退速度
        }
        else if(( dis > 20) && ( dis <= 35 ))           //距离大于25且小于30CM小车前进
        {
            TargetVel = 25;                                             //前进速度
        }
        else
        {
            TargetVel = 0;
        }
    }
    else if( Mode == 2)                                  //超声波避障模式
    {
            TargetVel = 30;
    }

    return TargetVel;
}

/*
 * 函数名：shut_down
 * 描述  ：关闭电机检测
 * 输入  ：angle小车当前角度，Target达到设置的目标角度关闭电机停止
 * 返回  ：关闭输出标志位
 */
uint8_t shut_down(float angle,float Target)
{
    uint8_t shut_flag = 0;
    if(f_abs(angle)>Target)                        						//取角度绝对值判断是否大于停止角度
    {
        shut_flag = 1;                                                  //关闭输出标志置位
    }
    return shut_flag;
}

static uint8_t theCarStopFlag = 0;
/*
 * 函数名：PickUpTheCar
 * 描述  ：小车被拿起
 * 输入  ：angle当前角度，leftEncoder左轮编码器值，rightEncoder右轮编码器值
 * 返回  ：是否成功拿起
 */
static uint8_t PickUpisOK = 0 ;
uint8_t PickUpTheCar(float carAngle , int leftEncoder , int rightEncoder)
{
    static uint16_t pickUpTime = 0 ;
    /* 小车被拿起后双轮高速转动  */
    if( leftEncoder >=37  || leftEncoder <=-37  ||  rightEncoder >=37  || rightEncoder <=-37  )
    {
        pickUpTime++;
        if(pickUpTime>200)                                   //转动时长：2s
        {
            pickUpTime = 0;
            PickUpisOK = 1;
        }
    }

    return PickUpisOK ;
}
/*
 * 函数名：PutDownTheCar
 * 描述  ：小车被放下
 * 输入  ：angle当前角度，leftEncoder左轮编码器值，rightEncoder右轮编码器值
 * 返回  ：是否成功放下
 */
uint8_t PutDownTheCar(float carAngle , int leftEncoder , int rightEncoder)
{
    static uint16_t isOK = 0 ;
    static uint16_t PutDownTime = 0;

    if(theCarStopFlag==0)                                   //放置误检
        return 0;
    /* 小车两轮静止，并且放置的角度在平衡位置10度附近 */
    if( int_abs(leftEncoder) ==0 &&  int_abs(rightEncoder) == 0 && (f_abs(carAngle) <= 10.0f) )
    {
        PutDownTime++;
        if(PutDownTime>200)                                 //放下持续约2S后平衡
        {
            PutDownTime = 0;
            isOK = 1;
        }
    }
    else
    {
        PutDownTime = 0;
        isOK = 0;
    }

    return isOK;
}

//**************************     PID控制    ******************************、
/*
 * 描述  ：角度PD控制器
 * 输入  ：angle当前角度，gyro当前角速度
 * 返回  ：PID控制输出
 */
int ctr_bal(float angle,float gyro)
{
    ctr.exp[0] = 0;                                                     //期望角度
    ctr.bais[0] = (float)(angle - ctr.exp[0]);                          //角度偏差
    ctr.balance = bal.kp * ctr.bais[0] + gyro * bal.kd;                 //角度平衡控制

    return ctr.balance;
}
int CarStepForwardOrBackward;
/*
 * 描述  ：速度控制
 * 输入  ：encoder_left编码器值A，encoder_right编码器值B
 * 返回  ：PID控制器输出
 */
int ctr_vel(int encoder_left,int encoder_right)
{
    static float encoder_cur,encoder_last;

    remote.ForwardOrBackward = remoteCarForwardOrBackward(mode);

    encoder_last = ((encoder_left) + (encoder_right)) - 0;              //速度误差
    encoder_cur = encoder_cur * 0.8 + encoder_last * 0.2;               //对偏差进行低通滤波
    ctr.bais[1] += encoder_cur;                                         //偏差累加和

    ctr.bais[1] = ctr.bais[1] - remote.ForwardOrBackward;               //遥控控制前后方向

    if(ctr.bais[1] > 10000)     ctr.bais[1] = 10000;                    //限幅
    if(ctr.bais[1] <-10000)	    ctr.bais[1] =-10000;
    ctr.velocity = encoder_cur * vel.kp + ctr.bais[1] * vel.ki;         //速度控制

    return ctr.velocity;
}



//转向控制
int CarStepLeftOrRight;
/*
 * 描述  ：转向控制
 * 输入  ：encoder_left编码器A的值，encoder_right编码器B的值，gyro_z转向的Z轴角速度
 * 返回  ：控制输出
 */
int ctr_turn(int turn, float gyro_z)
{
    static int remoteTurOut;

    remote.LeftOrRight=turn;
    remoteTurOut = (int)(-remote.LeftOrRight * tur.kp);
    i_limit(&remoteTurOut,2500);                        //输出速度范围限制，不能太快（最大2500）
    if(mode==2)
    {
        if( dis <= 25 )                 //距离小于10CM小车后退
        {
            ctr.turn = remoteTurOut + gyro_z * tur.kd;
        }
        else ctr.turn = 0;
    }
    else ctr.turn = 0;
    return ctr.turn;
}


/* 功能  ：设置PWM占空比，限幅
 * 描述  ：控制两路PWM输出
 * 输入  ：两路pwm
 */
uint32_t pwm_1=0,duty_1=0;
uint32_t pwm_2=0,duty_2=0;
void motor_pwm_out(uint16_t pwm1,uint16_t pwm2)
{
    pwm_1 = to_limit(pwm1,0,7199);
    pwm_2 = to_limit(pwm2,0,7199);
    R_GPT_DutyCycleSet(&g_pwm1_ctrl, pwm_1, GPT_IO_PIN_GTIOCA);
    R_GPT_DutyCycleSet(&g_pwm2_ctrl, pwm_2, GPT_IO_PIN_GTIOCA);

}
/*
 * 描述  ：控制输出
 */
void _ctr_out(void)
{
		uint8_t i;

    if(PickUpTheCar(att.rol,hall1count,hall2count) == 1)//小车被拿起检测
    {
        theCarStopFlag = 1;								//停止运行标志
    }
    if(PutDownTheCar(att.rol,hall1count,hall2count))    //小车被放下检测
    {
        theCarStopFlag = 0;								//运行标志
    }
    if( shut_down(att.rol,70.0f) || theCarStopFlag == 1)
    {

        PickUpisOK = 0;
        for( i = 0 ; i < 3 ; i++) //清除所有偏差
        {
            ctr.out[i] = 0;
            ctr.bais[i] = 0;
        }
        ctr.motor[0] = 0;
        ctr.motor[1] = 0;
    }
    else
    {
        ctr.motor[0] = ctr.out[0];
        ctr.motor[1] = ctr.out[1];
    }
    if(  theCarStopFlag == 0)
        motor_pwm_out(ctr.motor[0],ctr.motor[1]);  //正常输出
    else
        motor_pwm_out(0,0);                        //关闭电机
}

/*
 * 函数名：dir_config
 * 描述  ：电机转向控制
 * 输入  ：motor1电机1的pwm值首地址，motor2电机2的pwm值得首地址
 * 返回  ：
 */
void dir_config(int *motor1,int *motor2)
{
    if(*motor1<0) AIN1_HIGH, AIN2_LOW;
    else          AIN1_LOW, AIN2_HIGH;
    *motor1 = int_abs(*motor1);

    if(*motor2<0) BIN1_LOW, BIN2_HIGH;
    else          BIN1_HIGH, BIN2_LOW;
    *motor2 = int_abs(*motor2);
}
